/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import java.awt.Graphics2D;
import java.util.ArrayList;

/**
 *
 * @author Carlos Moisés Araldi Maciel
 */
public class Jogador extends Elemento {

     protected int cima; // código da tecla para mover para cima
     protected int baixo;// código da tecla para mover para baixo
     protected int direita;// código da tecla para mover para direita
     protected int esquerda;// código da tecla para mover para esquerda
     protected int atirar;// código da tecla para atirar
     private ArrayList<Bomba> bomba = new ArrayList<Bomba>();
     private int bombasSimultaneas; // número inteiro que determina a quantidade de bombas simultâneas do jogador
     private int raioExplosao; // número inteiro que determina a quantidade de blocos adicionais no qual o raio da explosão vai alcançar
     private int vidas; // determina a quantidade de vidas de um jogador
     private int PosXInicial;
     private int PosYInicial;

     /**
      *
      * @param pai referencia a area de Desenho
      * @param imagemNome - Nome da imagem que sera' usada para representar o jogador
      * @param bombaNome  - Nome da imagem que sera' usada para representar a bomba deste jogador
      * @param mX Linha na matriz
      * @param mY Coluna na matriz
      * @param c codigo da tecla que sera' usado para mover para cima
      * @param b codigo da tecla que sera' usado para mover para baixo
      * @param d codigo da tecla que sera' usado para mover para esquerda
      * @param e codigo da tecla que sera' usado para mover para direita
      * @param a codigo da tecla que sera' usado para atirar
      */
     public Jogador(JogoTimer pai, String imagemNome, String bombaNome,
             int mx, int my, int c, int b, int d, int e, int a) {
          super(pai, imagemNome, mx, my);
          this.icone = this.carregarImagem(this.imagemNome);
          this.cima = c;
          this.baixo = b;
          this.direita = d;
          this.esquerda = e;
          this.atirar = a;
          this.bombasSimultaneas = 1;
          this.raioExplosao = 1;
          this.vidas = 3;
          this.PosXInicial = this.posX;
          this.PosYInicial = this.posY;
     }

     /**
      * Desenha a imagem no contexto gráfico do Pai
      * @param g
      */
     @Override
     public void desenhar(Graphics2D g) {
          //desenhando o icone do Jogador na Tela
          g.drawImage(this.icone, this.posX, this.posY, this.jogo);
     }

     /**
      * Atualiza coordenadas, posicao na matriz e a bomba
      */
     @Override
     public synchronized void atualizar() {
          int destinoLin = matLinha; // linha destino apos o movimento
          int destinoCol = matColuna; // coluna destino apos o movimento
          int destinoX = posX; // coordenada X destino na area de desenho
          int destinoY = posY; // coordenada Y destino na area de desenho

          if (jogo.framePrincipal.teclado.teclaApertada(this.cima)) {
               destinoY = posY - this.velocidade;
               destinoLin--;
          } else if (jogo.framePrincipal.teclado.teclaApertada(this.baixo)) {
               destinoY = posY + this.velocidade;
               destinoLin++;
          } else if (jogo.framePrincipal.teclado.teclaApertada(this.esquerda)) {
               destinoX = posX - this.velocidade;
               destinoCol--;
          } else if (jogo.framePrincipal.teclado.teclaApertada(this.direita)) {
               destinoX = posX + this.velocidade;
               destinoCol++;
          }

          // se estiver a celula destino estiver com upgrade, também pode mover o personagem pra la'
          if (jogo.temUpgrade(destinoLin, destinoCol)) {
               matLinha = destinoLin;
               matColuna = destinoCol;
               posX = destinoX;
               posY = destinoY;
               if (jogo.tipoUpgrade(destinoLin, destinoCol) == 2) {
                    this.raioExplosao++;
                    this.jogo.escreverNoStatus("Aumentou raio de explosão ", this);
               }
               if (jogo.tipoUpgrade(destinoLin, destinoCol) == 3) {
                    if (this.raioExplosao > 1) {
                         this.raioExplosao--;
                         this.jogo.escreverNoStatus("Diminuiu raio de explosão ", this);
                    }
               }
               if (jogo.tipoUpgrade(destinoLin, destinoCol) == 0) {
                    this.bombasSimultaneas++;
                    this.jogo.escreverNoStatus("Aumentou quantidade de bombas ", this);
               }
               if (jogo.tipoUpgrade(destinoLin, destinoCol) == 1) {
                    if (this.bombasSimultaneas > 1) {
                         this.bombasSimultaneas--;
                         this.jogo.escreverNoStatus("Diminuiu quantidade de bombas ", this);
                    }
               }
               jogo.cenario[destinoLin][destinoCol] = 2;
          }
          // verifica se na posição destino contém alguma bomba lançada pelo adversário
          if (jogo.estaLivre(destinoLin, destinoCol)) {
               if (this.refOutroJogador().bomba.isEmpty()) {
                    matLinha = destinoLin;
                    matColuna = destinoCol;
                    posX = destinoX;
                    posY = destinoY;
               } else {
                    for (Bomba b : this.refOutroJogador().bomba) {
                         if (b.posXBomba() != destinoX || b.posYBomba() != destinoY) {
                              matLinha = destinoLin;
                              matColuna = destinoCol;
                              posX = destinoX;
                              posY = destinoY;
                         }
                    }
               }
          }

          // Ao apertar o botão de atirar bomba, irá acionar a bomba.
          if (jogo.framePrincipal.teclado.teclaApertadaPrimeiraVez(this.atirar)) {
               this.soltarBomba();
          }
     }

     /**
      * Retorna o valor inteiro da quantidade de bloco adicionais que o raio de explosão irá alcançar.
      * @return
      */
     public int getRaioExplosao() {
          return raioExplosao;
     }

     /**
      * Retorna o valor inteiro da quantidade de bombas simultaneas que está disponível no momento.
      * @return
      */
     public int getBombasSimultaneas() {
          return bombasSimultaneas;
     }

     /**
      * Modifica a variável de bombas simultâneas. E utilizado ao soltar a bomba e ao adquirir upgrades responsáveis.
      * @param bombasSimultaneas
      */
     public void setBombasSimultaneas(int bombasSimultaneas) {
          this.bombasSimultaneas = bombasSimultaneas;
     }

     /**
      * Retorna o valor inteiro da quantidade de vidas que o jogador tem no momento.
      * @return
      */
     public int getVidas() {
          return vidas;
     }

     /**
      * Modifica a variável de vidas do jogador. E utilizado quando o jogador morre.
      * @param vidas
      */
     public void setVidas(int vidas) {
          this.vidas = vidas;
     }

     /**
      * Retorna a ArrayList das bombas do jogador.
      * @return
      */
     public synchronized ArrayList<Bomba> getBomba() {
          return bomba;
     }

     /**
      * Método reponsável por lançar a bomba no cenário. Caso não tenha mais bombas disponíveis retorna no status "Bombas esgotadas".
      */
     public synchronized void soltarBomba() {
          if (bombasSimultaneas > 0) {
               bomba.add(new Bomba(this.jogo, "bomba.png", this.posX, this.posY, this, this.refOutroJogador()));
          } else {
               this.jogo.escreverNoStatus("Bombas esgotadas!", this);
          }
     }

     /**
      * Método utilizado após explodir a bomba. Incrementa 1 na quantidade de bombas simultâneas e elimina a bomba da ArrayList de bombas do jogador.
      * @param b
      */
     public synchronized void liberarB(Bomba b) {

          this.setBombasSimultaneas(this.getBombasSimultaneas() + 1);

          for (int i = 0; i < bomba.size(); i++) {
               Bomba bb = bomba.get(i);
               if (bb.isExplodida()) { //Se bomba já foi explodida retorna true e elimina a bomba da ArrayList
                    bomba.remove(bb);
               }
          }
     }

     /**
      * Método responsável por resetar a posição do jogador e seus atributos. Recebe como parametro o jogador. Utilizado quando o jogador morre.
      * @param j
      */
     public void resetPosJogador(Jogador j) {
          //Jogador 1
          if (j == this.jogo.jog1) {
               this.jogo.jog1.bombasSimultaneas = 1 - this.jogo.jog1.bomba.size();
               this.jogo.jog1.raioExplosao = 1;
               this.posX = 40;
               this.posY = 40;
               this.matLinha = 1; //posição inicial do jogador 1
               this.matColuna = 1; //posição inicial do jogador 1
          }
          //Jogador 2
          if (j == this.jogo.jog2) {
               this.jogo.jog2.bombasSimultaneas = 1 - this.jogo.jog2.bomba.size();
               this.jogo.jog2.raioExplosao = 1;
               this.posX = 600;
               this.posY = 360;
               this.matLinha = 9; //posição inicial do jogador 2
               this.matColuna = 15; //posição inicial do jogador 2
          }
     }

     /**
      * Método responsável por retirar uma vida do jogador que recebeu como parametro e resetar sua posição.
      * @param j
      */
     public void jogadorPerdeuVida(Jogador j) {
          this.setVidas(this.getVidas() - 1);
          j.resetPosJogador(j);
     }

     /**
      * Retorna o outro jogador da sessão de jogo.
      * @return
      */
     public Jogador refOutroJogador() {
          if (this == this.jogo.jog1) {
               return this.jogo.jog2;
          }
          return this.jogo.jog1;
     }

     /**
      * Método utilizado por modificar os comandos do jogador.
      * @param dir
      * @param esq
      * @param cim
      * @param bai
      * @param bom
      */
     public void configurarTeclas(int dir, int esq, int cim, int bai, int bom) {
          this.direita = dir;
          this.esquerda = esq;
          this.cima = cim;
          this.baixo = bai;
          this.atirar = bom;
     }
}
