/**
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
 *
 *  Prof. Emerson Ribeiro de Mello (2011)
 *  mello@ifsc.edu.br
 */
package jogo;

import principal.JavaBomber;
import java.awt.event.ActionEvent;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.Random;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.Timer;

/**
 * Essa clase contém a lógica principal do Jogo. A atualização da tela é feita
 * através de o Timer do pacote Swing. Por este motivo, essa classe tem que
 * implementar a Interface ActionListener
 * Essa classe herda de Canvas, pois a classe Canvas é ideal para fazer desenhos
 * e permite usufrir o bufferStrategy para melhorar a taxa de atualização da tela
 *
 * @author Emerson Ribeiro de Mello
 */
public class JogoTimer extends Canvas implements ActionListener {

     public JavaBomber framePrincipal; // frame principal do jogo
     public Jogador jog1; // Jogador1
     public Jogador jog2; // Jogador2
     private Timer timer; // Timer responsável pela atualização do jogo
     public int larguraBloco; // Dimensao dos personabens, fundos. É 40
     public int colunas; // total de colunas na matriz do jogo
     public int linhas; // total de linhas na matriz do jogo
     private Image imgcenarios[]; // contém as imagens de fundo do cenário
     private Random upgrade = new Random();
     private Random upgrade2 = new Random();
     private Random geradorLimiteUpgrade = new Random();
     private int limiteUpgrade; // Variável que limita a quantidade de upgrades que será lançado no cenário durante a sessão
     private int contadorUpgrades; // Variável que conta a quantidade de upgrades que já foram lançados no cenário
     private int taxaDeAtualizacao; // a cada N milisegundos a tela é redesenhada
     private int qtdeJogosGanhosJog1;
     private int qtdeJogosGanhosJog2;
     // 0 = ferro
     // 1 = tijolo
     // 2 = vazio
     // 3 = up_bomba
     // 4 = down_bomba
     // 5 = up_raio
     // 6 = down_raio
     // 7 = bomba (campo sem imagem) - somente para indicar que personagem nao pode passar por cima da bomba
     // 8 =
     // 9 =
     // ...
     public int[][] cenario = new int[10][16];

     public JogoTimer(JavaBomber p) {
          super();
          this.framePrincipal = p;
          this.setBackground(Color.GRAY);

          this.setIgnoreRepaint(true); // para melhorar a velocidade do redesenho
          this.larguraBloco = 40;
          this.colunas = 17;
          this.linhas = 11;
          this.taxaDeAtualizacao = 120;
          this.imgcenarios = new Image[8];
          this.imgcenarios[0] = (new ImageIcon(this.getClass().getResource("/imagens/ferro.png"))).getImage();
          this.imgcenarios[1] = (new ImageIcon(this.getClass().getResource("/imagens/tijolo.png"))).getImage();

          //upgrades
          this.imgcenarios[3] = (new ImageIcon(this.getClass().getResource("/imagens/up_bomba.png"))).getImage();
          this.imgcenarios[4] = (new ImageIcon(this.getClass().getResource("/imagens/down_bomba.png"))).getImage();
          this.imgcenarios[5] = (new ImageIcon(this.getClass().getResource("/imagens/up_raio.png"))).getImage();
          this.imgcenarios[6] = (new ImageIcon(this.getClass().getResource("/imagens/down_raio.png"))).getImage();

          this.contadorUpgrades = 0;

     }

     /**
      * Método invocado para iniciar a partida
      */
     public void comecaOJogo() {
          this.fabricarCenario();
          // criando o Timer com taxa de atualização de N milisegundos
          this.timer = new Timer(taxaDeAtualizacao, this);
          // criando 2 buffer para acelerar a taxa de desenho
          createBufferStrategy(2);

          // KeyEvent possui codigos para as teclas. Por exemplo, para obter
          // o codigo da tecla 'seta para cima' basta invocar KeyEvent.VK_UP
          // Veja o uso do KeyEvent na criacao dos jogadores abaixo

          // criando Jogador 1
          this.jog1 = new Jogador(this, "azul.png", "bomba.png", 1, 1,
                  framePrincipal.getJog1Up(), framePrincipal.getJog1Down(),
                  framePrincipal.getJog1Right(), framePrincipal.getJog1Left(), framePrincipal.getJog1Bomb());

          // criando Jogador 2
          this.jog2 = new Jogador(this, "vermelho.png", "bomba.png", 15, 9,
                  framePrincipal.getJog2Up(), framePrincipal.getJog2Down(),
                  framePrincipal.getJog2Right(), framePrincipal.getJog2Left(), framePrincipal.getJog2Bomb());

          // Disparando o timer para começar a atualização do Loop principal do Jogo
          // Todo o conteúdo do método actionPerformed dessa classe será executado
          // a cada 60 milisegundos.
          this.limiteUpgrade = 1 + geradorLimiteUpgrade.nextInt(20); //limite da quantidade de upgrades que poderão ser lançados no cenário de 1 até 20
          this.escreverNoStatus("Quantidade de possíveis upgrades no cenário: " + this.limiteUpgrade);
          this.timer.start();
     }

     /**
      * Principal METODO do JOGO, pois e' aqui que fica a logica que fara':
      * - processamento da logica do jogo (morreu, disparou, etc..)
      * - atualizar as coordenadas dos elementos na tela para o proximo redesenho
      * - redesenhar os elementos na tela
      *
      * Este método é invocado a cada N milisegundos pelo Timer
      * @param e
      */
     @Override
     public synchronized void actionPerformed(ActionEvent e) {
          this.framePrincipal.teclado.poll(); // processa os eventos no teclado
          this.processarLogica(); // atualiza as coordenadas dos elementos
          this.renderizar(); // desenha os elementos no buffer de desenho -- EM UM BUFFER SECUNDÁRIO
          this.desenhar(); // faz a troca do buffer de desenho pelo buffer de apresentacao


          // As linhas abaixo só servem para informar a quantidade de vidas do jogador. Representados pela quantidade de icones do proprio jogador
          //Jogador 1
          if (jog1 != null) {
               this.framePrincipal.jTextFieldJogosGanhosJog1.setText("" + this.qtdeJogosGanhosJog1);
               if (this.jog1.getVidas() == 3) {
                    framePrincipal.jLIconeVida1Jog1.setVisible(true);
                    framePrincipal.jLIconeVida2Jog1.setVisible(true);
                    framePrincipal.jLIconeVida3Jog1.setVisible(true);
               }
               if (this.jog1.getVidas() == 2) {
                    framePrincipal.jLIconeVida1Jog1.setVisible(true);
                    framePrincipal.jLIconeVida2Jog1.setVisible(true);
                    framePrincipal.jLIconeVida3Jog1.setVisible(false);
               }
               if (this.jog1.getVidas() == 1) {
                    framePrincipal.jLIconeVida1Jog1.setVisible(true);
                    framePrincipal.jLIconeVida2Jog1.setVisible(false);
                    framePrincipal.jLIconeVida3Jog1.setVisible(false);
               }

          }
          // Jogador 2
          if (jog2 != null) {
               this.framePrincipal.jTextFieldJogosGanhosJog2.setText("" + this.qtdeJogosGanhosJog2);
               if (this.jog2.getVidas() == 3) {
                    framePrincipal.jLIconeVida1Jog2.setVisible(true);
                    framePrincipal.jLIconeVida2Jog2.setVisible(true);
                    framePrincipal.jLIconeVida3Jog2.setVisible(true);
               }
               if (this.jog2.getVidas() == 2) {
                    framePrincipal.jLIconeVida1Jog2.setVisible(true);
                    framePrincipal.jLIconeVida2Jog2.setVisible(true);
                    framePrincipal.jLIconeVida3Jog2.setVisible(false);
               }
               if (this.jog2.getVidas() == 1) {
                    framePrincipal.jLIconeVida1Jog2.setVisible(true);
                    framePrincipal.jLIconeVida2Jog2.setVisible(false);
                    framePrincipal.jLIconeVida3Jog2.setVisible(false);
               }
          }
     }

     /**
      * Atualiza as coordenadas/estados de todos os elementos presentes no jogo
      * No caso, somente 2 jogadores, mas aqui pode-se colocar qualquer outro
      * elemento que venha a ser incluido no jogo
      */
     public synchronized void processarLogica() {
          //Chama o atualizar dos elementos
          if (this.jog1 != null) {
               this.jog1.atualizar();
          }
          if (this.jog2 != null) {
               this.jog2.atualizar();
          }
     }

     /**
      * Renderiza no buffer secundário todos os elementos já com suas coordenadas/estados
      * atualizados.
      */
     public synchronized void renderizar() {
          Graphics g = getBufferStrategy().getDrawGraphics();
          Graphics2D g2 = (Graphics2D) g;
          // Gerando um gradiente que ficara' como cor de fundo
          GradientPaint gp = new GradientPaint(300, 300, new Color(255, 210, 174), 50, 50, new Color(170, 200, 90), true);
          g2.setPaint(gp);
          g2.fillRect(0, 0, getWidth(), getHeight());

          // desenhar cenario de fundo
          this.renderizarCenario(g2);

          // desenhando os elementos nao nulos
          if (this.jog1 != null) {
               this.jog1.desenhar(g2);
               for (int i = 0; i < this.jog1.getBomba().size(); i++) {
                    Bomba b = this.jog1.getBomba().get(i);
                    if (b != null) {
                         b.desenhar(g2);
                    }
               }
               // Se a quantidade de vidas de um jogador chegar a 0 finaliza o jogo.
               if (this.jog1.getVidas() == 0) {
                    this.escreverNoStatus("Jogador 2 é o vencedor desta partida!!!");
                    this.qtdeJogosGanhosJog2++; //Acrescenta no placar a quantidade de jogos já ganhos.
                    this.reiniciarJogo();
               }
          }
          if (this.jog2 != null) {
               this.jog2.desenhar(g2);
               for (int i = 0; i < this.jog2.getBomba().size(); i++) {
                    Bomba b = this.jog2.getBomba().get(i);
                    if (b != null) {
                         b.desenhar(g2);
                    }
               }
               // Se a quantidade de vidas de um jogador chegar a 0 finaliza o jogo.
               if (this.jog2.getVidas() == 0) {
                    this.escreverNoStatus("Jogador 1 é o vencedor desta partida!!!");
                    this.qtdeJogosGanhosJog1++; //Acrescenta no placar a quantidade de jogos já ganhos.
                    this.reiniciarJogo();
               }
          }

          //liberando os contextos gráficos
          g.dispose();
          g2.dispose();

     }

     //}
     /**
      * Efetuando a troca das areas de desenho e apresentacao.
      * No metodo redenrizar os elemtos graficos foram desenhados em um buffer
      * secundario, agora iremos fazer com que esse buffer secundario torne-se
      * primario, ou seja, que apareca de fato na tela
      */
     public synchronized void desenhar() {
          if (!getBufferStrategy().contentsLost()) {
               getBufferStrategy().show();
          }
     }

     /**
      * Fim de jogo/rodada
      */
     public void finalizar() {
          this.jog1 = null;
          this.jog2 = null;
          this.timer.stop();
     }

     /**
      * Desenha o cenário com as paredes de tijolos e de ferro
      * @param g contexto gráfico
      */
     public void renderizarCenario(Graphics2D g) {
          for (int i = 0; i < 11; i++) { // linhas
               for (int j = 0; j < 17; j++) { // colunas
                    if (this.cenario[i][j] != 2) { // se não for vazio, desenhe
                         g.drawImage(this.imgcenarios[this.cenario[i][j]], this.larguraBloco * j, this.larguraBloco * i, this.framePrincipal);
                    }
               }
          }
     }


     /**
      * Método responsável por substituir o bloco de tijolo por um bloco livre ou um upgrade. Se a quantidade de upgrades for alcançado previsto no limite não lançará mais upgrades no cenário
      * @param lin
      * @param col
      */
     public void explodeTijolo(int lin, int col) {
          if (this.contadorUpgrades < this.limiteUpgrade) { //Verifica se já chegou ao limite de upgrades no cenário
               if (1 + upgrade2.nextInt(6) == 1) { //Caso não, faz um sorteio de um número aleatório de 1 a 6, caso for 1 lança um upgrade no local (16,6%)
                    this.cenario[lin][col] = 3 + upgrade.nextInt(4);
                    this.contadorUpgrades++; //Acrescenta ao contador de upgrades já lançados
               } else {
                    //Caso não foi sorteado um upgrade, deixa o espaço vazio (bloco livre)
                    this.cenario[lin][col] = 2;
               }
          } else {
               //Caso já chegou ao limite de upgrades, deixa o espaço vazio (bloco livre)
               this.cenario[lin][col] = 2;
          }

     }

     /**
      * Verifica se na posicao lin,col da matriz tem um espaço vazio
      * @param lin numero da linha
      * @param col numero da coluna
      * @return true se estiver vazio, false caso contrario
      */
     public boolean estaLivre(int lin, int col) {
          return cenario[lin][col] == 2; // se for == 2, entao ira' retornar true, false caso contrario
     }

     /**
      * Método responsável por informar se na posição informada tem upgrade ou não. Caso sim retorna true
      * @param lin
      * @param col
      * @return
      */
     public boolean temUpgrade(int lin, int col) {
          return cenario[lin][col] == 3 || cenario[lin][col] == 4 || cenario[lin][col] == 5 || cenario[lin][col] == 6; // se for ==3,4,5,6 retorna true
     }

     /**
      * Método responsável por retonar o tipo de upgrade na posição informada.
      * @param lin
      * @param col
      * @return
      */
     public int tipoUpgrade(int lin, int col) {
          switch (cenario[lin][col]) {
               case 3:
                    // upgrade "up bomba"
                    return 0;
               case 4:
                    // downgrade "down_bomba"
                    return 1;
               case 5:
                    // upgrade "up_raio"
                    return 2;
               case 6:
                    // downgrade "down_raio"
                    return 3;

          }
          return 0;

     }
     /**
      * Método responsável por retonar o tipo de upgrade na posição informada.
      * @param lin
      * @param col
      * @return
      */
     public boolean isTijolo(int lin, int col) { // se for tijolo retorna true
          if (this.cenario[lin][col] == 1) {
               return true;
          }
          return false;
     }

     /**
      * Indica em que posicao na tela encontrase o inicio dessa o retangulo que
      * compreende uma celula
      * @param celula posicao na matriz que deseja-se descobrir
      * @return Coordenada na tela
      */
     public int celulaParaPixel(int celula) {
          return larguraBloco * celula;

     }

     /**
      * Indica qual celula o pixel esta' contido na matriz
      * @param pixel
      * @return
      */
     public int pixelParaCelula(int pixel) {
          return pixel / larguraBloco;
     }

     /**
      * Pergunta se deseja jogar novamente. Pode reiniciar o jogo, ou finaliza-lo de acordo com a resposta do usuário
      */
     public void reiniciarJogo() {

          if (JOptionPane.showConfirmDialog(this, "Deseja jogar novamente??") == 0) {
               //Finaliza o jogo atual e reinicia
               this.finalizar();
               this.comecaOJogo();

          } else {
               //Finaliza e fecha o jogo
               JOptionPane.showMessageDialog(this, "Jogo Terminado");
               this.finalizar();
               System.exit(0);
          }
     }

     /**
      * Controi aleatóriamente o cenário, composto por espaços livres, blocos de tijolo ou blocos de metal
      */
     public void fabricarCenario() {
          Random tijolo_vazio = new Random();
          int[][] cenarioPrincipal = {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
               {0, 2, 2, 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 2, 2, 0},
               {0, 2, 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 2, 0},
               {0, 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 2, 0},
               {0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0},
               {0, 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 0},
               {0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0},
               {0, 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 0},
               {0, 2, 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 1 + tijolo_vazio.nextInt(2), 0, 2, 0},
               {0, 2, 2, 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 1 + tijolo_vazio.nextInt(2), 2, 2, 0},
               {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
          };
          this.cenario = cenarioPrincipal;
     }

     /**
      * Pausa o jogo atual. Trava todos os comandos e só é possível continuar acionando o botão correspondente.
      */
     public void pausarJogo() {
          this.timer.stop();
     }

     /**
      * Continua o jogo "pausado".
      */
     public void despausarJogo() {
          this.timer.start();
     }

     /**
      * Método responsável por escrever no status do jogo. Recebe os parametros de qual jogador e a mensagem a ser exibida
      * @param s
      * @param j
      */
     public void escreverNoStatus(String s, Jogador j) {

          if (j == this.jog1) {
               this.framePrincipal.jTextAreaStatus.append("Bomberman Azul :.........." + s + "\n");
          } else if (j == this.jog2) {
               this.framePrincipal.jTextAreaStatus.append("Bomberman Vermelho :.........." + s + "\n");
          }
     }

     /**
      * Método responsável por escrever no status do jogo. Modo geral. Não indica de qual jogador é a mensagem.
      * @param s
      */
     public void escreverNoStatus(String s) {
          this.framePrincipal.jTextAreaStatus.append(s + "\n");
     }
}
